Class: Karafka::ActiveJob::JobOptionsContract
- Inherits:
-
Contracts::Base
- Object
- Core::Contractable::Contract
- Contracts::Base
- Karafka::ActiveJob::JobOptionsContract
- Defined in:
- lib/karafka/active_job/job_options_contract.rb
Overview
Note:
We keep this in the Karafka::ActiveJob namespace instead of Karafka::Contracts as we want to keep ActiveJob related Karafka components outside of the core Karafka code and all in the same place
Contract for validating the options that can be altered with #karafka_options per job class