Class: Karafka::Instrumentation::Callbacks::ConsumerGroups::Decorator

Inherits:
Core::Monitoring::StatisticsDecorator
  • Object
show all
Defined in:
lib/karafka/instrumentation/callbacks/consumer_groups/decorator.rb

Overview

Default statistics decorator used for consumer group statistics decoration.

This is a thin subclass (rather than referencing Karafka::Core::Monitoring::StatisticsDecorator directly) so it can be swapped out via config.internal.statistics.consumer_groups.decorator_class for a custom decorator (for example one that also enriches or corrects specific statistics values) without having to alter the callback that uses it.