Module: Karafka::Pro::Processing::Strategies::Aj::DlqFtrLrjMom

Includes:
DlqMom, FtrMom, LrjMom
Defined in:
lib/karafka/pro/processing/strategies/aj/dlq_ftr_lrj_mom.rb

Overview

ActiveJob enabled DLQ enabled Filtering enabled Long-Running Job enabled Manual offset management enabled

Constant Summary collapse

FEATURES =

Features for this strategy

%i[
  active_job
  dead_letter_queue
  filtering
  long_running_job
  manual_offset_management
].freeze

Instance Method Summary collapse

Methods included from Lrj::Mom

#handle_before_schedule_consume, #handle_revoked

Methods included from Default

#handle_before_consume, #handle_before_schedule_consume, #handle_before_schedule_tick, #handle_consume, #handle_revoked, #handle_tick, #mark_as_consumed, #mark_as_consumed!, #mark_in_memory, #mark_in_transaction, #mark_with_transaction, #store_offset_metadata, #transaction

Methods included from Karafka::Processing::Strategies::Default

#commit_offsets, #commit_offsets!, #handle_before_consume, #handle_consume, #handle_eofed, #handle_idle, #handle_initialized, #handle_revoked, #handle_shutdown, #handle_wrap, #mark_as_consumed, #mark_as_consumed!

Methods included from Karafka::Processing::Strategies::Base

#handle_before_consume, #handle_consume, #handle_idle, #handle_revoked, #handle_shutdown

Methods included from Dlq::Mom

#mark_after_dispatch?

Methods included from Dlq::Default

#apply_dlq_flow, #build_dlq_message, #dispatch_if_needed_and_mark_as_consumed, #dispatch_in_a_transaction?, #dispatch_to_dlq, #dispatch_to_dlq?, #find_skippable_message, #mark_after_dispatch?, #mark_as_consumed, #mark_as_consumed!, #mark_dispatched_to_dlq

Methods included from Ftr::Default

#handle_idle, #handle_post_filtering

Instance Method Details

#handle_after_consumeObject

This strategy assumes we do not early break on shutdown as it has VP



56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
81
82
# File 'lib/karafka/pro/processing/strategies/aj/dlq_ftr_lrj_mom.rb', line 56

def handle_after_consume
  coordinator.on_finished do
    if coordinator.success?
      coordinator.pause_tracker.reset

      if coordinator.filtered? && !revoked?
        handle_post_filtering

        # :seek and :pause are fully handled by handle_post_filtering
        # For :skip we still need to resume the LRJ MAX_PAUSE_TIME pause
        return unless coordinator.filter.action == :skip
      elsif !revoked?
        # no need to check for manual seek because AJ consumer is internal and
        # fully controlled by us
        seek(seek_offset, false, reset_offset: false)
      end

      resume
    else
      apply_dlq_flow do
        skippable_message, = find_skippable_message
        dispatch_to_dlq(skippable_message) if dispatch_to_dlq?
        mark_dispatched_to_dlq(skippable_message)
      end
    end
  end
end