Module: Karafka::Pro::Processing::Strategies::Dlq::FtrLrj

Includes:
Lrj, Lrj::Ftr
Included in:
FtrLrjVp
Defined in:
lib/karafka/pro/processing/strategies/dlq/ftr_lrj.rb

Overview

Dead-Letter Queue enabled Filtering enabled Long-Running Job enabled

Constant Summary collapse

FEATURES =

Features for this strategy

%i[
  dead_letter_queue
  filtering
  long_running_job
].freeze

Constants included from Lrj::Default

Lrj::Default::MAX_PAUSE_TIME

Instance Method Summary collapse

Methods included from Lrj::Default

#handle_before_schedule_consume, #handle_revoked, #synchronize

Methods included from Karafka::Pro::Processing::Strategies::Default

#handle_before_consume, #handle_before_schedule_consume, #handle_before_schedule_tick, #handle_consume, #handle_revoked, #handle_tick, #mark_as_consumed, #mark_as_consumed!, #mark_in_memory, #mark_in_transaction, #mark_with_transaction, #store_offset_metadata, #transaction

Methods included from Karafka::Processing::Strategies::Default

#commit_offsets, #commit_offsets!, #handle_before_consume, #handle_consume, #handle_eofed, #handle_idle, #handle_initialized, #handle_revoked, #handle_shutdown, #handle_wrap, #mark_as_consumed, #mark_as_consumed!

Methods included from Karafka::Processing::Strategies::Base

#handle_before_consume, #handle_consume, #handle_idle, #handle_revoked, #handle_shutdown

Methods included from Ftr::Default

#handle_idle, #handle_post_filtering

Methods included from Default

#apply_dlq_flow, #build_dlq_message, #dispatch_if_needed_and_mark_as_consumed, #dispatch_in_a_transaction?, #dispatch_to_dlq, #dispatch_to_dlq?, #find_skippable_message, #mark_after_dispatch?, #mark_as_consumed, #mark_as_consumed!, #mark_dispatched_to_dlq

Instance Method Details

#handle_after_consumeObject

This is one of more complex cases. We need to ensure, that we always resume (inline or via paused backoff) and we need to make sure we dispatch to DLQ when needed. Because revocation on LRJ can happen any time, we need to make sure we do not dispatch to DLQ when error happens but we no longer own the assignment. Throttling is another factor that has to be taken into consideration on the successful path



56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
81
82
83
84
# File 'lib/karafka/pro/processing/strategies/dlq/ftr_lrj.rb', line 56

def handle_after_consume
  coordinator.on_finished do |last_group_message|
    if coordinator.success?
      coordinator.pause_tracker.reset

      return if coordinator.manual_pause?

      mark_as_consumed(last_group_message) unless revoked?

      if coordinator.filtered? && !revoked?
        handle_post_filtering

        # :seek and :pause are fully handled by handle_post_filtering
        # For :skip we still need to resume the LRJ MAX_PAUSE_TIME pause
        return unless coordinator.filter.action == :skip
      elsif !revoked? && !coordinator.manual_seek?
        seek(seek_offset, false, reset_offset: false)
      end

      resume
    else
      apply_dlq_flow do
        return resume if revoked?

        dispatch_if_needed_and_mark_as_consumed
      end
    end
  end
end