Module: Karafka::Pro::Processing::Strategies::Dlq::FtrMom
- Includes:
- Default, Ftr::Default
- Included in:
- Aj::DlqFtrMom, FtrMomVp
- Defined in:
- lib/karafka/pro/processing/strategies/dlq/ftr_mom.rb
Overview
-
DLQ
-
Ftr
-
Mom
Constant Summary collapse
- FEATURES =
Features for this strategy
%i[ dead_letter_queue filtering manual_offset_management ].freeze
Instance Method Summary collapse
-
#handle_after_consume ⇒ Object
On mom we do not mark, throttling and seeking as in other strategies.
-
#mark_after_dispatch? ⇒ Boolean
Should we mark given message as consumed after dispatch.
Methods included from Default
#apply_dlq_flow, #build_dlq_message, #dispatch_if_needed_and_mark_as_consumed, #dispatch_in_a_transaction?, #dispatch_to_dlq, #dispatch_to_dlq?, #find_skippable_message, #mark_as_consumed, #mark_as_consumed!, #mark_dispatched_to_dlq
Methods included from Karafka::Pro::Processing::Strategies::Default
#handle_before_consume, #handle_before_schedule_consume, #handle_before_schedule_tick, #handle_consume, #handle_revoked, #handle_tick, #mark_as_consumed, #mark_as_consumed!, #mark_in_memory, #mark_in_transaction, #mark_with_transaction, #store_offset_metadata, #transaction
Methods included from Karafka::Processing::Strategies::Default
#commit_offsets, #commit_offsets!, #handle_before_consume, #handle_consume, #handle_eofed, #handle_idle, #handle_initialized, #handle_revoked, #handle_shutdown, #handle_wrap, #mark_as_consumed, #mark_as_consumed!
Methods included from Karafka::Processing::Strategies::Base
#handle_before_consume, #handle_consume, #handle_idle, #handle_revoked, #handle_shutdown
Methods included from Ftr::Default
#handle_idle, #handle_post_filtering
Instance Method Details
#handle_after_consume ⇒ Object
On mom we do not mark, throttling and seeking as in other strategies
51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 72 73 74 |
# File 'lib/karafka/pro/processing/strategies/dlq/ftr_mom.rb', line 51 def handle_after_consume coordinator.on_finished do return if revoked? if coordinator.success? coordinator.pause_tracker.reset return if coordinator.manual_pause? handle_post_filtering else apply_dlq_flow do , _marked = dispatch_to_dlq() if dispatch_to_dlq? if mark_after_dispatch? mark_dispatched_to_dlq() else self.seek_offset = .offset + 1 end end end end end |
#mark_after_dispatch? ⇒ Boolean
Please note, this is the opposite behavior than in case of AOM strategies.
Returns should we mark given message as consumed after dispatch. For MOM strategies if user did not explicitly tell us to mark, we do not mark. Default is ‘nil`, which means `false` in this case. If user provided alternative value, we go with it.
82 83 84 85 86 |
# File 'lib/karafka/pro/processing/strategies/dlq/ftr_mom.rb', line 82 def mark_after_dispatch? return false if topic.dead_letter_queue.mark_after_dispatch.nil? topic.dead_letter_queue.mark_after_dispatch end |