Module: Karafka::Pro::Processing::Strategies::Dlq::LrjMom

Includes:
Lrj
Included in:
FtrLrjMom, LrjMomVp
Defined in:
lib/karafka/pro/processing/strategies/dlq/lrj_mom.rb

Overview

Strategy for supporting DLQ with Mom and LRJ enabled

Constant Summary collapse

FEATURES =

Features for this strategy

%i[
  dead_letter_queue
  long_running_job
  manual_offset_management
].freeze

Constants included from Lrj::Default

Lrj::Default::MAX_PAUSE_TIME

Instance Method Summary collapse

Methods included from Lrj::Default

#handle_before_schedule_consume, #handle_revoked, #synchronize

Methods included from Karafka::Pro::Processing::Strategies::Default

#handle_before_consume, #handle_before_schedule_consume, #handle_before_schedule_tick, #handle_consume, #handle_revoked, #handle_tick, #mark_as_consumed, #mark_as_consumed!, #mark_in_memory, #mark_in_transaction, #mark_with_transaction, #store_offset_metadata, #transaction

Methods included from Karafka::Processing::Strategies::Default

#commit_offsets, #commit_offsets!, #handle_before_consume, #handle_consume, #handle_eofed, #handle_idle, #handle_initialized, #handle_revoked, #handle_shutdown, #handle_wrap, #mark_as_consumed, #mark_as_consumed!

Methods included from Karafka::Processing::Strategies::Base

#handle_before_consume, #handle_consume, #handle_idle, #handle_revoked, #handle_shutdown

Methods included from Default

#apply_dlq_flow, #build_dlq_message, #dispatch_if_needed_and_mark_as_consumed, #dispatch_in_a_transaction?, #dispatch_to_dlq, #dispatch_to_dlq?, #find_skippable_message, #mark_as_consumed, #mark_as_consumed!, #mark_dispatched_to_dlq

Instance Method Details

#handle_after_consumeObject

LRJ standard flow after consumption with DLQ dispatch and no offset management



50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
# File 'lib/karafka/pro/processing/strategies/dlq/lrj_mom.rb', line 50

def handle_after_consume
  coordinator.on_finished do |last_group_message|
    if coordinator.success?
      coordinator.pause_tracker.reset

      return if coordinator.manual_pause?

      unless revoked? || coordinator.manual_seek?
        seek(last_group_message.offset + 1, false, reset_offset: false)
      end

      resume
    else
      apply_dlq_flow do
        return resume if revoked?

        skippable_message, _marked = find_skippable_message
        dispatch_to_dlq(skippable_message) if dispatch_to_dlq?

        if mark_after_dispatch?
          mark_dispatched_to_dlq(skippable_message)
        else
          self.seek_offset = skippable_message.offset + 1
        end
      end
    end
  end
end

#mark_after_dispatch?Boolean

Note:

Please note, this is the opposite behavior than in case of AOM strategies.

Returns should we mark given message as consumed after dispatch. For MOM strategies if user did not explicitly tell us to mark, we do not mark. Default is ‘nil`, which means `false` in this case. If user provided alternative value, we go with it.

Returns:

  • (Boolean)

    should we mark given message as consumed after dispatch. For MOM strategies if user did not explicitly tell us to mark, we do not mark. Default is ‘nil`, which means `false` in this case. If user provided alternative value, we go with it.



85
86
87
88
89
# File 'lib/karafka/pro/processing/strategies/dlq/lrj_mom.rb', line 85

def mark_after_dispatch?
  return false if topic.dead_letter_queue.mark_after_dispatch.nil?

  topic.dead_letter_queue.mark_after_dispatch
end