Class: Karafka::Pro::Routing::Features::ConsumerGroups::Multiplexing::Config

Inherits:
Struct
  • Object
show all
Defined in:
lib/karafka/pro/routing/features/consumer_groups/multiplexing/config.rb

Overview

Multiplexing configuration

Instance Attribute Summary collapse

Instance Method Summary collapse

Instance Attribute Details

#activeObject Also known as: active?

Returns the value of attribute active

Returns:

  • (Object)

    the current value of active



38
39
40
# File 'lib/karafka/pro/routing/features/consumer_groups/multiplexing/config.rb', line 38

def active
  @active
end

#bootObject

Returns the value of attribute boot

Returns:

  • (Object)

    the current value of boot



38
39
40
# File 'lib/karafka/pro/routing/features/consumer_groups/multiplexing/config.rb', line 38

def boot
  @boot
end

#maxObject

Returns the value of attribute max

Returns:

  • (Object)

    the current value of max



38
39
40
# File 'lib/karafka/pro/routing/features/consumer_groups/multiplexing/config.rb', line 38

def max
  @max
end

#minObject

Returns the value of attribute min

Returns:

  • (Object)

    the current value of min



38
39
40
# File 'lib/karafka/pro/routing/features/consumer_groups/multiplexing/config.rb', line 38

def min
  @min
end

#scale_delayObject

Returns the value of attribute scale_delay

Returns:

  • (Object)

    the current value of scale_delay



38
39
40
# File 'lib/karafka/pro/routing/features/consumer_groups/multiplexing/config.rb', line 38

def scale_delay
  @scale_delay
end

Instance Method Details

#dynamic?Boolean

Returns true if we are allowed to upscale and downscale.

Returns:

  • (Boolean)

    true if we are allowed to upscale and downscale



49
50
51
# File 'lib/karafka/pro/routing/features/consumer_groups/multiplexing/config.rb', line 49

def dynamic?
  min < max
end