Class: Karafka::Pro::Routing::Features::ConsumerGroups::OffsetMetadata
- Inherits:
-
Base
- Object
- Routing::Features::Base
- Base
- Karafka::Pro::Routing::Features::ConsumerGroups::OffsetMetadata
- Defined in:
- lib/karafka/pro/routing/features/consumer_groups/offset_metadata.rb,
lib/karafka/pro/routing/features/consumer_groups/offset_metadata/topic.rb,
lib/karafka/pro/routing/features/consumer_groups/offset_metadata/config.rb,
lib/karafka/pro/routing/features/consumer_groups/offset_metadata/contracts/topic.rb
Overview
Offset Metadata Support with a custom deserializer
Defined Under Namespace
Modules: Contracts, Topic Classes: Config
Class Method Summary collapse
-
.post_setup(_config) ⇒ Object
If needed installs the needed listener and initializes tracker.
Methods inherited from Routing::Features::Base
activate, load_all, post_setup_all, pre_setup_all
Class Method Details
.post_setup(_config) ⇒ Object
If needed installs the needed listener and initializes tracker
42 43 44 45 46 47 48 49 50 51 52 |
# File 'lib/karafka/pro/routing/features/consumer_groups/offset_metadata.rb', line 42 def post_setup(_config) Karafka::App.monitor.subscribe("app.running") do # Initialize the tracker prior to becoming multi-threaded Karafka::Processing::ConsumerGroups::InlineInsights::Tracker.instance # Subscribe to the statistics reports and collect them Karafka.monitor.subscribe( Karafka::Pro::Processing::ConsumerGroups::OffsetMetadata::Listener.new ) end end |