Class: Karafka::Pro::Routing::Features::ConsumerGroups::Patterns
- Inherits:
-
Base
- Object
- Routing::Features::Base
- Base
- Karafka::Pro::Routing::Features::ConsumerGroups::Patterns
- Defined in:
- lib/karafka/pro/routing/features/consumer_groups/patterns.rb,
lib/karafka/pro/routing/features/consumer_groups/patterns/topic.rb,
lib/karafka/pro/routing/features/consumer_groups/patterns/config.rb,
lib/karafka/pro/routing/features/consumer_groups/patterns/topics.rb,
lib/karafka/pro/routing/features/consumer_groups/patterns/builder.rb,
lib/karafka/pro/routing/features/consumer_groups/patterns/pattern.rb,
lib/karafka/pro/routing/features/consumer_groups/patterns/detector.rb,
lib/karafka/pro/routing/features/consumer_groups/patterns/patterns.rb,
lib/karafka/pro/routing/features/consumer_groups/patterns/consumer_group.rb,
lib/karafka/pro/routing/features/consumer_groups/patterns/contracts/topic.rb,
lib/karafka/pro/routing/features/consumer_groups/patterns/contracts/pattern.rb,
lib/karafka/pro/routing/features/consumer_groups/patterns/contracts/consumer_group.rb
Overview
Dynamic topics builder feature.
Allows you to define patterns in routes that would then automatically subscribe and start consuming new topics.
This feature works by injecting a topic that represents a regexp subscription (matcher) that at the same time holds the builder block for full config of a newly detected topic.
We inject a virtual topic to hold settings but also to be able to run validations during boot to ensure consistency of the pattern base setup.
Defined Under Namespace
Modules: Builder, ConsumerGroup, Contracts, Topic, Topics Classes: Config, Detector, Pattern, Patterns