Module: Karafka::Pro::Routing::Features::OffsetMetadata::Topic

Defined in:
lib/karafka/pro/routing/features/offset_metadata/topic.rb

Overview

Commit Metadata API extensions

Instance Method Summary collapse

Instance Method Details

#initializeObject

This method calls the parent class initializer and then sets up the extra instance variable to nil. The explicit initialization to nil is included as an optimization for Ruby’s object shapes system, which improves memory layout and access performance.



57
58
59
60
# File 'lib/karafka/pro/routing/features/offset_metadata/topic.rb', line 57

def initialize(...)
  super
  @offset_metadata = nil
end

#offset_metadata(cache: true, deserializer: STRING_DESERIALIZER) ⇒ Config

Returns this feature config.

Parameters:

  • cache (Boolean) (defaults to: true)

    should we cache the response until rebalance

  • deserializer (#call) (defaults to: STRING_DESERIALIZER)

    deserializer that will get raw data and should return deserialized metadata

Returns:

  • (Config)

    this feature config



66
67
68
69
70
71
72
# File 'lib/karafka/pro/routing/features/offset_metadata/topic.rb', line 66

def (cache: true, deserializer: STRING_DESERIALIZER)
  @offset_metadata ||= Config.new(
    active: true,
    cache: cache,
    deserializer: deserializer
  )
end

#offset_metadata?true

Returns is offset metadata active (it always is).

Returns:

  • (true)

    is offset metadata active (it always is)



75
76
77
# File 'lib/karafka/pro/routing/features/offset_metadata/topic.rb', line 75

def offset_metadata?
  .active?
end

#to_hHash

Returns topic with all its native configuration options plus offset metadata settings.

Returns:

  • (Hash)

    topic with all its native configuration options plus offset metadata settings



81
82
83
84
85
# File 'lib/karafka/pro/routing/features/offset_metadata/topic.rb', line 81

def to_h
  super.merge(
    offset_metadata: .to_h
  ).freeze
end