Class: Karafka::Pro::ScheduledMessages::State

Inherits:
Object
  • Object
show all
Defined in:
lib/karafka/pro/scheduled_messages/state.rb

Overview

Represents the loading/bootstrapping state of the given topic partition

Bootstrapping can be in the following states:

  • fresh - when we got an assignment but we did not load the schedule yet

  • loading - when we are in the process of bootstrapping the daily state and we consume historical messages to build the needed schedules.

  • loaded - state in which we finished loading all the schedules and we can dispatch messages when the time comes and we can process real-time incoming schedules and changes to schedules as they appear in the stream.

  • shutdown - the states are no longer available as the consumer has shut down

Instance Method Summary collapse

Constructor Details

#initializeState

Initializes the state as fresh



56
57
58
# File 'lib/karafka/pro/scheduled_messages/state.rb', line 56

def initialize
  @state = "fresh"
end

Instance Method Details

#to_sString

Returns current state string representation.

Returns:

  • (String)

    current state string representation



71
72
73
# File 'lib/karafka/pro/scheduled_messages/state.rb', line 71

def to_s
  @state
end