Module: Karafka::Processing::Jobs
- Defined in:
- lib/karafka/processing/jobs/base.rb
Overview
Namespace for all the jobs that are supposed to run in workers.
Defined Under Namespace
Classes: Base
Namespace for all the jobs that are supposed to run in workers.
Classes: Base