Class: Karafka::Routing::Features::ConsumerGroups::Eofed
- Defined in:
- lib/karafka/routing/features/consumer_groups/eofed.rb,
lib/karafka/routing/features/consumer_groups/eofed/topic.rb,
lib/karafka/routing/features/consumer_groups/eofed/config.rb,
lib/karafka/routing/features/consumer_groups/eofed/contracts/topic.rb
Overview
Namespace for feature allowing to enable the ‘#eofed` jobs. We do not enable it always because users may only be interested in fast eofed yielding without running the `#eofed` operation at all. This saves on empty cycles of running pointless empty jobs.
Defined Under Namespace
Modules: Contracts, Topic Classes: Config