Module: Karafka::Routing::Features::Declaratives::Contracts

Defined in:
lib/karafka/routing/features/declaratives/contracts/topic.rb

Overview

Contracts used by the routing Expander to validate topic declarations after draw

Constant Summary collapse

Topic =

Delegates to the canonical contract in the Declaratives namespace. Required here because the Expander looks up ‘scope::Contracts::Topic`.

Karafka::Declaratives::Contracts::Topic