Module: Karafka::Routing::Features::Pausing

Defined in:
lib/karafka/routing/features/pausing.rb,
lib/karafka/routing/features/pausing/config.rb

Overview

Namespace holding the pause (backoff) routing configuration. There is no OSS pausing feature to activate here - the backoff behavior is part of the topic itself and its settings default to the global config.pause.*.

Defined Under Namespace

Classes: Config