Module: Karafka::Setup::Dsl
- Included in:
- App
- Defined in:
- lib/karafka/setup/dsl.rb
Overview
Note:
Despite providing methods, everything is still persisted and fetched
Dsl for allowing to work with the configuration from the Karafka::App from the Karafka::Setup::Config
Instance Method Summary collapse
-
#config ⇒ Karafka::Config
Config instance.
-
#setup ⇒ Object
Sets up the whole configuration.
Instance Method Details
#config ⇒ Karafka::Config
Returns config instance.
15 16 17 |
# File 'lib/karafka/setup/dsl.rb', line 15 def config Setup::Config.config end |
#setup ⇒ Object
Sets up the whole configuration
10 11 12 |
# File 'lib/karafka/setup/dsl.rb', line 10 def setup(&) Setup::Config.setup(&) end |