Module: Karafka::Pro::Processing::Strategies::Dlq::Ftr
- Includes:
- Default, Ftr::Default
- Included in:
- FtrVp
- Defined in:
- lib/karafka/pro/processing/strategies/dlq/ftr.rb
Overview
-
DLQ
-
Ftr
Constant Summary collapse
- FEATURES =
Features for this strategy
%i[ dead_letter_queue filtering ].freeze
Instance Method Summary collapse
-
#handle_after_consume ⇒ Object
DLQ flow is standard here, what is not, is the success component where we need to take into consideration the filtering.
Methods included from Default
#apply_dlq_flow, #build_dlq_message, #dispatch_if_needed_and_mark_as_consumed, #dispatch_in_a_transaction?, #dispatch_to_dlq, #dispatch_to_dlq?, #find_skippable_message, #mark_after_dispatch?, #mark_as_consumed, #mark_as_consumed!, #mark_dispatched_to_dlq
Methods included from Karafka::Pro::Processing::Strategies::Default
#handle_before_consume, #handle_before_schedule_consume, #handle_before_schedule_tick, #handle_consume, #handle_revoked, #handle_tick, #mark_as_consumed, #mark_as_consumed!, #mark_in_memory, #mark_in_transaction, #mark_with_transaction, #store_offset_metadata, #transaction
Methods included from Karafka::Processing::Strategies::Default
#commit_offsets, #commit_offsets!, #handle_before_consume, #handle_consume, #handle_eofed, #handle_idle, #handle_initialized, #handle_revoked, #handle_shutdown, #handle_wrap, #mark_as_consumed, #mark_as_consumed!
Methods included from Karafka::Processing::Strategies::Base
#handle_before_consume, #handle_consume, #handle_idle, #handle_revoked, #handle_shutdown
Methods included from Ftr::Default
#handle_idle, #handle_post_filtering
Instance Method Details
#handle_after_consume ⇒ Object
DLQ flow is standard here, what is not, is the success component where we need to take into consideration the filtering
50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 |
# File 'lib/karafka/pro/processing/strategies/dlq/ftr.rb', line 50 def handle_after_consume coordinator.on_finished do || return if revoked? if coordinator.success? coordinator.pause_tracker.reset return if coordinator.manual_pause? mark_as_consumed() handle_post_filtering else apply_dlq_flow do dispatch_if_needed_and_mark_as_consumed end end end end |