Class: Karafka::Pro::Routing::Features::Pausing
- Inherits:
-
Base
- Object
- Routing::Features::Base
- Base
- Karafka::Pro::Routing::Features::Pausing
- Defined in:
- lib/karafka/pro/routing/features/pausing.rb,
lib/karafka/pro/routing/features/pausing/topic.rb,
lib/karafka/pro/routing/features/pausing/config.rb,
lib/karafka/pro/routing/features/pausing/contracts/topic.rb
Overview
Feature allowing for a per-route reconfiguration of the pausing strategy It can be useful when different topics should have different backoff policies
Defined Under Namespace
Constant Summary collapse
- Config =
Config for pausing feature.
By convention each routing feature exposes a
Config. The pause value object is defined in OSS (Karafka::Routing::Features::Pausing::Config) because the per-topic#pausereader defaults to the global config there, so the feature points itsConfigat that shared class rather than redefining it. Karafka::Routing::Features::Pausing::Config