Module: Karafka::Pro::Routing::Features::ScheduledMessages::Topic
- Defined in:
- lib/karafka/pro/routing/features/scheduled_messages/topic.rb
Overview
Topic extensions to be able to check if given topic is a scheduled messages topic Please note, that this applies to both the schedules topic and logs topic
Instance Method Summary collapse
-
#initialize ⇒ Object
This method calls the parent class initializer and then sets up the extra instance variable to nil.
- #scheduled_messages(active = false) ⇒ Object
-
#scheduled_messages? ⇒ Boolean
Is this an ActiveJob topic.
-
#to_h ⇒ Hash
messages namespace settings.
Instance Method Details
#initialize ⇒ Object
This method calls the parent class initializer and then sets up the extra instance variable to nil. The explicit initialization to nil is included as an optimization for Ruby’s object shapes system, which improves memory layout and access performance.
35 36 37 38 |
# File 'lib/karafka/pro/routing/features/scheduled_messages/topic.rb', line 35 def initialize(...) super @scheduled_messages = nil end |
#scheduled_messages(active = false) ⇒ Object
41 42 43 |
# File 'lib/karafka/pro/routing/features/scheduled_messages/topic.rb', line 41 def (active = false) @scheduled_messages ||= Config.new(active: active) end |
#scheduled_messages? ⇒ Boolean
Returns is this an ActiveJob topic.
46 47 48 |
# File 'lib/karafka/pro/routing/features/scheduled_messages/topic.rb', line 46 def .active? end |
#to_h ⇒ Hash
messages namespace settings
52 53 54 55 56 |
# File 'lib/karafka/pro/routing/features/scheduled_messages/topic.rb', line 52 def to_h super.merge( scheduled_messages: .to_h ).freeze end |