Module: Karafka::Pro::Routing::Features::Swarm::Topic
- Defined in:
- lib/karafka/pro/routing/features/swarm/topic.rb
Overview
Topic swarm API extensions
Instance Method Summary collapse
-
#active? ⇒ Boolean
Should this topic be active.
-
#initialize ⇒ Object
This method calls the parent class initializer and then sets up the extra instance variable to nil.
-
#swarm(nodes: (0...Karafka::App.config.swarm.nodes)) ⇒ Object
Allows defining swarm routing topic settings.
-
#swarm? ⇒ true
Swarm setup is always true.
-
#to_h ⇒ Hash
Topic with all its native configuration options plus swarm.
Instance Method Details
#active? ⇒ Boolean
Returns should this topic be active. In the context of swarm it is only active when swarm routing setup does not limit nodes on which it should operate.
84 85 86 87 88 89 90 |
# File 'lib/karafka/pro/routing/features/swarm/topic.rb', line 84 def active? node = Karafka::App.config.swarm.node return super unless node super && swarm.nodes.include?(node.id) end |
#initialize ⇒ Object
This method calls the parent class initializer and then sets up the extra instance variable to nil. The explicit initialization to nil is included as an optimization for Ruby’s object shapes system, which improves memory layout and access performance.
42 43 44 45 |
# File 'lib/karafka/pro/routing/features/swarm/topic.rb', line 42 def initialize(...) super @swarm = nil end |
#swarm(nodes: (0...Karafka::App.config.swarm.nodes)) ⇒ Object
Allows defining swarm routing topic settings
73 74 75 |
# File 'lib/karafka/pro/routing/features/swarm/topic.rb', line 73 def swarm(nodes: (0...Karafka::App.config.swarm.nodes)) @swarm ||= Config.new(active: true, nodes: nodes) end |
#swarm? ⇒ true
Returns swarm setup is always true. May not be in use but is active.
78 79 80 |
# File 'lib/karafka/pro/routing/features/swarm/topic.rb', line 78 def swarm? swarm.active? end |
#to_h ⇒ Hash
Returns topic with all its native configuration options plus swarm.
93 94 95 96 97 |
# File 'lib/karafka/pro/routing/features/swarm/topic.rb', line 93 def to_h super.merge( swarm: swarm.to_h ).freeze end |