Class: Karafka::Processing::ConsumerGroups::Jobs::Revoked
- Inherits:
-
Jobs::Base
- Object
- Jobs::Base
- Karafka::Processing::ConsumerGroups::Jobs::Revoked
- Defined in:
- lib/karafka/processing/consumer_groups/jobs/revoked.rb
Overview
Job that runs the revoked operation when we lose a partition on a consumer that lost it.
Direct Known Subclasses
Karafka::Pro::Processing::ConsumerGroups::Jobs::RevokedNonBlocking
Instance Attribute Summary
Attributes inherited from Jobs::Base
Instance Method Summary collapse
-
#before_schedule ⇒ Object
Runs code prior to scheduling this revoked job.
-
#call ⇒ Object
Runs the revoking job via an executor.
- #initialize(executor) ⇒ Revoked constructor
Methods inherited from Jobs::Base
#after_call, #before_call, #finish!, #finished?, #non_blocking?, #wrap
Constructor Details
#initialize(executor) ⇒ Revoked
16 17 18 19 |
# File 'lib/karafka/processing/consumer_groups/jobs/revoked.rb', line 16 def initialize(executor) @executor = executor super() end |
Instance Method Details
#before_schedule ⇒ Object
Runs code prior to scheduling this revoked job
22 23 24 |
# File 'lib/karafka/processing/consumer_groups/jobs/revoked.rb', line 22 def before_schedule executor.before_schedule_revoked end |
#call ⇒ Object
Runs the revoking job via an executor.
27 28 29 |
# File 'lib/karafka/processing/consumer_groups/jobs/revoked.rb', line 27 def call executor.revoked end |