Module: Karafka::Processing::ConsumerGroups::Strategies::Base
- Defined in:
- lib/karafka/processing/consumer_groups/strategies/base.rb
Overview
Base strategy that should be included in each strategy, just to ensure the API
Instance Method Summary collapse
-
#handle_after_consume ⇒ Object
Post-consumption handling.
-
#handle_before_consume ⇒ Object
What should happen before we kick in the processing.
-
#handle_consume ⇒ Object
What should happen in the processing.
-
#handle_idle ⇒ Object
Idle run handling.
-
#handle_revoked ⇒ Object
Revocation handling.
-
#handle_shutdown ⇒ Object
Shutdown handling.
Instance Method Details
#handle_after_consume ⇒ Object
Post-consumption handling
41 42 43 |
# File 'lib/karafka/processing/consumer_groups/strategies/base.rb', line 41 def handle_after_consume raise NotImplementedError, "Implement in a subclass" end |
#handle_before_consume ⇒ Object
What should happen before we kick in the processing
31 32 33 |
# File 'lib/karafka/processing/consumer_groups/strategies/base.rb', line 31 def handle_before_consume raise NotImplementedError, "Implement in a subclass" end |
#handle_consume ⇒ Object
What should happen in the processing
36 37 38 |
# File 'lib/karafka/processing/consumer_groups/strategies/base.rb', line 36 def handle_consume raise NotImplementedError, "Implement in a subclass" end |
#handle_idle ⇒ Object
Idle run handling
46 47 48 |
# File 'lib/karafka/processing/consumer_groups/strategies/base.rb', line 46 def handle_idle raise NotImplementedError, "Implement in a subclass" end |
#handle_revoked ⇒ Object
Revocation handling
51 52 53 |
# File 'lib/karafka/processing/consumer_groups/strategies/base.rb', line 51 def handle_revoked raise NotImplementedError, "Implement in a subclass" end |
#handle_shutdown ⇒ Object
Shutdown handling
56 57 58 |
# File 'lib/karafka/processing/consumer_groups/strategies/base.rb', line 56 def handle_shutdown raise NotImplementedError, "Implement in a subclass" end |