Module: Karafka::Processing::ConsumerGroups::Strategies::Base
- Defined in:
- lib/karafka/processing/consumer_groups/strategies/base.rb
Overview
Base strategy that should be included in each strategy, just to ensure the API
Instance Method Summary collapse
-
#handle_after_consume ⇒ Object
Post-consumption handling.
-
#handle_before_consume ⇒ Object
What should happen before we kick in the processing.
-
#handle_consume ⇒ Object
What should happen in the processing.
-
#handle_idle ⇒ Object
Idle run handling.
-
#handle_revoked ⇒ Object
Revocation handling.
-
#handle_shutdown ⇒ Object
Shutdown handling.
Instance Method Details
#handle_after_consume ⇒ Object
Post-consumption handling
43 44 45 |
# File 'lib/karafka/processing/consumer_groups/strategies/base.rb', line 43 def handle_after_consume raise NotImplementedError, "Implement in a subclass" end |
#handle_before_consume ⇒ Object
What should happen before we kick in the processing
33 34 35 |
# File 'lib/karafka/processing/consumer_groups/strategies/base.rb', line 33 def handle_before_consume raise NotImplementedError, "Implement in a subclass" end |
#handle_consume ⇒ Object
What should happen in the processing
38 39 40 |
# File 'lib/karafka/processing/consumer_groups/strategies/base.rb', line 38 def handle_consume raise NotImplementedError, "Implement in a subclass" end |
#handle_idle ⇒ Object
Idle run handling
48 49 50 |
# File 'lib/karafka/processing/consumer_groups/strategies/base.rb', line 48 def handle_idle raise NotImplementedError, "Implement in a subclass" end |
#handle_revoked ⇒ Object
Revocation handling
53 54 55 |
# File 'lib/karafka/processing/consumer_groups/strategies/base.rb', line 53 def handle_revoked raise NotImplementedError, "Implement in a subclass" end |
#handle_shutdown ⇒ Object
Shutdown handling
58 59 60 |
# File 'lib/karafka/processing/consumer_groups/strategies/base.rb', line 58 def handle_shutdown raise NotImplementedError, "Implement in a subclass" end |